package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;

/**
 *	Class Cell.
 *	<p>
 *	This class represents the GUI component view for an individual cell in a Sudoku puzzle.<br>
 *	<p>
 *	Written by:	Roger West
 *				University of Illinois at Springfield
 *				Jun 26, 2014
 */
public class Cell extends JPanel
{
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Attributes
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/** Default height for this Cell */
	public final static int DEFAULT_HEIGHT = 64;
	
	/** Default width for this Cell */
	public final static int DEFAULT_WIDTH = 64;
	
	/** Font to use for value */
	private final static Font CELL_FONT = new Font("Verdana", Font.BOLD, 18);
	
	/** Value to display for unassigned cells */
	public final static String UNASSIGNED_VALUE = "";
	
	/** Font color to use for pre-assigned cell values */
	public final static Color CLR_PREASSIGNED = Color.BLACK;
	
	/** Font color to use for cell values assigned during the search */
	public final static Color CLR_SEARCH_ASSIGNED = Color.BLUE;
	
	/** Background color to use for the most recently assigned Cell */
	public final static Color CLR_MOST_RECENTLY_ASSIGNED = Color.ORANGE;
	
	/** Background color for all other Cells */
	public final static Color CLR_DEFAULT_BG_COLOR = Color.WHITE;
	
	/** Layout to use for this Cell */
	private BorderLayout layout = new BorderLayout();
	
	/** Label for storing value for this Cell */
	private JLabel lbl_value;
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Constructors
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Create a new Cell, located at the specified row and column, and containing the specified
	 *	initial value.
	 *
	 *	@param	row				row position for this Cell
	 *	@param	column			column position for this Cell
	 *	@param	initialValue	initial value to display in this Cell
	 */
	public Cell(int row, int column, String initialValue)
	{
		// set name to allow accessing specific Cells
		this.setName("" + row + "," + column);
		
		// initially, no search, so all cells use default background color
		this.setBackground(CLR_DEFAULT_BG_COLOR);
		
		// set layout
		setLayout(layout);
		
		// set preferred size
		this.setPreferredSize(new Dimension(DEFAULT_HEIGHT, DEFAULT_WIDTH));
		
		// initialize internal components of this Cell
		initComponents(initialValue);
	}
	
	
	/**
	 *	Initialize the internal components of this Cell.
	 *
	 *	@param	initialValue	the initial value to display for this Cell
	 */
	private void initComponents(String initialValue)
	{
		// set properties for value label
		lbl_value = new JLabel(initialValue.toString());
		lbl_value.setFont(CELL_FONT);
		lbl_value.setHorizontalAlignment(JTextField.CENTER);
		lbl_value.setForeground(CLR_PREASSIGNED);
		lbl_value.setBackground(CLR_DEFAULT_BG_COLOR);
		
		// add value label to this Cell
		this.add(lbl_value);
	}
	
	
	/**
	 *	Set the value for this Cell.
	 *
	 *	@param	newValue	the value to set
	 */
	public void setValue(String newValue)
	{
		lbl_value.setText(newValue.toString());
	}
	
	
	/**
	 *	Set the value for this Cell to reflect an unassigned Cell.
	 */
	public void setValueAsUnassigned()
	{
		lbl_value.setText(UNASSIGNED_VALUE);
	}
	
	
	/**
	 *	Reset this Cell. Background color will be reset to default color, and value
	 *	displayed will reflect an unassigned cell.
	 */
	public void resetCell()
	{
		lbl_value.setForeground(CLR_PREASSIGNED);
		resetBackground();
		lbl_value.setText(UNASSIGNED_VALUE);
	}
	
	
	/**
	 *	Return whether this Cell was assigned a value by a search.
	 *
	 *	@return	true, if this Cell was assigned a value by a search
	 *			false, if this Cell was pre-assigned as part of a puzzle's initial state, or if
	 *				   this Cell is unassigned
	 */
	public boolean wasSearchAssigned()
	{
		return lbl_value.getForeground().equals(CLR_SEARCH_ASSIGNED);
	}
	
	
	/**
	 *	Set the foreground font color of this Cell to reflect this Cell has had its
	 *	value assigned by a search.
	 */
	public void setAsSearchAssigned()
	{
		lbl_value.setForeground(CLR_SEARCH_ASSIGNED);
	}
	
	
	/**
	 *	Set the background color of this Cell to reflect the fact that this cell was
	 *	the most recently assigned cell.
	 */
	public void setAsCurrent()
	{
		this.setBackground(CLR_MOST_RECENTLY_ASSIGNED);
	}
	
	
	/**
	 *	Set this Cell's background color to indicate it is not the current Cell of 
	 *	the search.
	 */
	public void setAsNotCurrent()
	{
		this.setBackground(CLR_DEFAULT_BG_COLOR);
	}
	
	
	/**
	 *	Reset the background color of this Cell to the default background color.
	 */
	public void resetBackground()
	{
		lbl_value.setBackground(CLR_DEFAULT_BG_COLOR);
	}
	
	
	/**
	 *	Return the value being displayed by this Cell.
	 *
	 *	@return	the value currently being displayed by this Cell
	 */
	public String getValue()
	{
		return lbl_value.getText();
	}

}
