package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.LinkedList;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import javax.swing.SwingWorker;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;

import sudokuCSP.BacktrackingSearch;
import sudokuCSP.BruteForceVariableSelector;
import sudokuCSP.MRVVariableSelector;
import sudokuCSP.SudokuCSPAgent;
import sudokuCSP.SudokuPuzzle;
import sudokuCSP.SudokuPuzzleReader;
import sudokuCSP.SudokuVariable;
import sudokuCSP.VariableSelector;

/**
 *	Class MainFrame.
 *	<p>
 *	This class is the main GUI window for the application. This window appears upon program<br>
 *	launch. Through this window, it is possible to:<br>
 *		1)	Load a Sudoku puzzle from a data file<br>
 *		2)	Set the variable selection algorithm (Brute Force vs. MRV)<br>
 *		3)	Initiate the solving of the loaded puzzle<br>
 *	<p>
 *	Written by:	Roger West
 *				University of Illinois at Springfield
 *				Jun 26, 2014
 */
public class MainFrame extends JFrame
{
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Attributes
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/** Panel containing the Sudoku puzzle grid. */
	private PuzzlePanel pnl_puzzle;
	
	/** Panel containing the UI controls. */
	private ControlPanel pnl_controlPanel;
	
	/** Panel for showing updates as the program is running. */
	private StatusBar pnl_statusBar;
	
	/** Puzzle width, in terms of number of columns. */
	private int puzzleWidth;
	
	/** Puzzle height, in terms of number of rows. */
	private int puzzleHeight;
	
	/** The working copy of the SudokuPuzzle being solved. */
	private SudokuPuzzle puzzle;
	
	/** The initial state of the SudokuPuzzle being solved. */
	private SudokuPuzzle initialState;
	
	/** SwingWorker thread to allow agent to solve puzzle in background and still allow
	 *	GUI to be updated as the agent progresses.
	 */
	private SwingWorker<Boolean, Void> worker;
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Constructors
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Create a new MainFrame based on the specified puzzle width & height.
	 *
	 *	@param	puzzleWidth		puzzle width, in terms of number of columns
	 *	@param	puzzleHeight	Puzzle height, in terms of number of rows
	 */
	public MainFrame(int puzzleWidth, int puzzleHeight)
	{
		super("Sudoku");
		
		// create anonymous listener to allow user to close window and end sim
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
				System.exit(0);
			}
		});
		
		// set component layout scheme
		getContentPane().setLayout(new BorderLayout());
		
		// set puzzle height & width
		this.puzzleWidth = puzzleWidth;
		this.puzzleHeight = puzzleHeight;
		
		// initialize internal components
		initComponents();
		
		// set the size of this MainFrame
		setSize(new Dimension(Cell.DEFAULT_WIDTH * puzzleWidth, Cell.DEFAULT_HEIGHT * puzzleHeight + 66));
		
		// make GUI visible
		setVisible(true);
		
		// force layout and validation of subcomponents
		validate();
	}
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Constructors
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Initialize internal components, and add them to the layout.
	 */
	private void initComponents()
	{
		// create the PuzzlePanel
		pnl_puzzle = new PuzzlePanel(puzzleWidth, puzzleHeight);
		
		for (int row = 0; row < puzzleWidth; row++)
		{
			for (int column = 0; column < puzzleHeight; column++)
			{
				Cell currentCell = new Cell(row, column, "");
				
				currentCell.setValueAsUnassigned();
				
				currentCell.setBorder(determineCellBorders(row, column));
				
				pnl_puzzle.addCell(currentCell);
			}
		}
		
		// create the ControlPanel
		pnl_controlPanel = new ControlPanel(Cell.DEFAULT_WIDTH * puzzleWidth, 40);
		
		// create the StatusBar
		pnl_statusBar = new StatusBar(new Dimension(Cell.DEFAULT_WIDTH * puzzleWidth, 32));
		clearStatus();
		
		// add components to layout
		getContentPane().add(pnl_controlPanel, BorderLayout.NORTH);
		getContentPane().add(pnl_puzzle, BorderLayout.CENTER);
		getContentPane().add(pnl_statusBar, BorderLayout.SOUTH);
	}
	
	
	/**
	 *	Determine and return the specific border to use for the Cell at the location specified
	 *	by row and column. A thin border should separate the Cells at a minimum, but thicker
	 *	borders are used selectively to highlight the boxes of the puzzle grid. A MatteBorder
	 *	is used for this, since it is the only built-in Border that allows different thicknesses
	 *	for each side of the Border.
	 * 
	 *	@param	row		the row of the Cell
	 *	@param	column	the column of the Cell
	 *
	 *	@return	a MatteBorder that 
	 */
	public Border determineCellBorders(int row, int column)
	{
		// the default Border used by most of the Cells
		Border b = BorderFactory.createMatteBorder(1, 1, 1, 1, Color.DARK_GRAY);
		
		// to facilitate distinguishing which Border to use, use an index created by
		// logically treating the 2D array as a 1D array, numbered from left to right,
		// top to bottom, beginning in the upper left corner (Cell 0 to Cell 80)
		int index = (row * puzzleWidth) + column;
		
		switch(index)
		{
			case 2:		case 5:
			case 11:	case 14:
			case 38:	case 41:
			case 65:	case 68:
			case 74:	case 77:
				b = BorderFactory.createMatteBorder(1, 1, 1, 3, Color.DARK_GRAY);
				break;
			
			case 3:		case 6:
			case 12:	case 15:
			case 39:	case 42:
			case 66:	case 69:
			case 75:	case 78:
				b = BorderFactory.createMatteBorder(1, 3, 1, 1, Color.DARK_GRAY);
				break;
			
			case 18:	case 19:	case 22:	case 25:	case 26:
			case 45:	case 46:	case 49:	case 52:	case 53:
				b = BorderFactory.createMatteBorder(1, 1, 3, 1, Color.DARK_GRAY);
				break;
			
			case 27:	case 28:	case 31:	case 34:	case 35:
			case 54:	case 55:	case 58:	case 61:	case 62:
				b = BorderFactory.createMatteBorder(3, 1, 1, 1, Color.DARK_GRAY);
				break;
			
			case 20:	case 23:
			case 47:	case 50:
				b = BorderFactory.createMatteBorder(1, 1, 3, 3, Color.DARK_GRAY);
				break;
			
			case 21:	case 24:
			case 48:	case 51:
				b = BorderFactory.createMatteBorder(1, 3, 3, 1, Color.DARK_GRAY);
				break;
			
			case 29:	case 32:
			case 56:	case 59:
				b = BorderFactory.createMatteBorder(3, 1, 1, 3, Color.DARK_GRAY);
				break;
			
			case 30:	case 33:
			case 57:	case 60:
				b = BorderFactory.createMatteBorder(3, 3, 1, 1, Color.DARK_GRAY);
				break;
		}
		
		return b;
	}
	
	
	/**
	 *	Set the SudokuPuzzle to be displayed.
	 *
	 *	@param	newPuzzle	the SudokuPuzzle to be displayed
	 */
	public void setPuzzle(SudokuPuzzle newPuzzle)
	{
		// reset PuzzlePanel
		pnl_puzzle.clearPuzzle();
		
		// populate grid
		for (int row = 0; row < newPuzzle.getAllCells().length; row++)
		{
			for (int column = 0; column < newPuzzle.getAllCells()[0].length; column++)
			{
				if (!newPuzzle.getAllCells()[row][column].getValue().equals(SudokuPuzzle.EMPTY_CELL_VALUE))
				{
					// populate pre-assigned value
					pnl_puzzle.setCellValue(row, column, newPuzzle.getAllCells()[row][column].getValue());
				}
				else
				{
					// populate unassigned cell
					pnl_puzzle.setCellValue(row, column, Cell.UNASSIGNED_VALUE);
				}
			}
		}
	}
	
	
	/**
	 *	Set the Cell at the specified row and column to the specified value.
	 *
	 *	@param	row		the row position of the Cell to set 
	 *	@param	column	the column position of the Cell to set
	 *	@param	value	the value to set
	 */
	public void setCellValue(int row, int column, String value)
	{
		// retrieve the Cell
		Cell cell = pnl_puzzle.getCellByRowColumn(row, column);
		
		// use font color to indicate a search-assigned Cell
		cell.setAsSearchAssigned();
		
		// set the background color of the Cell to indicate it is the current cell
		cell.setAsCurrent();
		
		// display the value
		cell.setValue(value);
	}
	
	
	/**
	 *	Set the Cell at the specified row and column to be unassigned, removing
	 *	any value that may be present.
	 *
	 *	@param	row		the row position of the Cell to set 
	 *	@param	column	the column position of the Cell to set
	 */
	public void setCellAsUnassigned(int row, int column)
	{
		// retrieve the Cell
		Cell cell = pnl_puzzle.getCellByRowColumn(row, column);
		
		// unassign the Cell
		cell.setValueAsUnassigned();
	}
	
	
	/**
	 *	Set the Cell at the specified row and column to be the current Cell.
	 *	The background color of the Cell will be changed to highlight the Cell.
	 *
	 *	@param	row		the row position of the Cell to set 
	 *	@param	column	the column position of the Cell to set
	 */
	public void setCellAsCurrent(int row, int column)
	{
		// retrieve the Cell
		Cell cell = pnl_puzzle.getCellByRowColumn(row, column);
		
		// set as current Cell
		cell.setAsCurrent();
	}
	
	
	/**
	 *	Set the Cell at the specified row and column to no longer be
	 *	the current Cell. The background color of the Cell will be
	 *	returned to the default color.
	 *
	 *	@param	row		the row position of the Cell to set 
	 *	@param	column	the column position of the Cell to set
	 */
	public void setCellAsNotCurrent(int row, int column)
	{
		Cell cell = pnl_puzzle.getCellByRowColumn(row, column);
		
		cell.setAsNotCurrent();
	}
	
	
	/**
	 *	Display an update to the status bar.
	 *
	 *	@param	newStatus	the text to display
	 */
	public void updateStatus(String newStatus)
	{
		pnl_statusBar.updateStatus(newStatus);
	}
	
	
	/**
	 *	Append information to the status bar.
	 *
	 *	@param	addendum	the text to append
	 */
	public void appendStatus(String addendum)
	{
		pnl_statusBar.updateStatus(pnl_statusBar.lbl_status.getText() + addendum);
	}
	
	
	/**
	 *	Clear the status bar of any specific puzzle-related information.
	 */
	public void clearStatus()
	{
		pnl_statusBar.updateStatus("Ready");
	}
	
	
	/**
	 *	Return the SwingWorker thread the agent is using to solve the puzzle.
	 *
	 *	@return	the agent's SwingWorker thread
	 */
	public SwingWorker<Boolean, Void> getAgentWorker()
	{
		return worker;
	}
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Inner Classes
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Inner class ControlPanel.
	 *	<p>
	 *	This class serves to contain the user controls; e.g., buttons, radio buttons, etc.,<br>
	 *	for the application.<br>
	 *	<p>
	 *	Written by:	Roger West
	 *				University of Illinois at Springfield
	 *				Jun 26, 2014
	 */
	class ControlPanel extends JPanel
	{
		
		///////////////////////////////////////////////////////////////////////////////////////////////
		//	Attributes
		///////////////////////////////////////////////////////////////////////////////////////////////
		
		/** Button for loading a puzzle from a data file. */
		private JButton btn_loadPuzzle;
		
		/** Radio button for choosing the brute force variable selector */
		private JRadioButton rbtn_bruteForceVariableSelector;
		
		/** Radio button for choosing the variable selector that uses the MRV heuristic*/
		private JRadioButton rbtn_MRVVariableSelector;
		
		/** ButtonGroup object to ensure only 1 variable selector can be chosen */
		private ButtonGroup bgrp_variableSelector;
		
		/** Button for initiating search to solve the puzzle. */
		private JButton btn_solvePuzzle;
		
		/** Button to reset a puzzle back to its initial state */
		private JButton btn_resetPuzzle;
		
		/** Button to cancel search in progress. */
		private JButton btn_cancelSearch;
		
		/** Font for button text */
		private Font fnt_buttons = new Font("Calibri", Font.BOLD, 12);
		
		/** Event handler for buttons */
		private ButtonHandler buttonHandler;
		
		
		///////////////////////////////////////////////////////////////////////////////////////////////
		//	Constructors
		///////////////////////////////////////////////////////////////////////////////////////////////
		
		/**
		 *	Create a new ControlPanel with the specified height and width, in pixels.
		 *
		 *	@param	height	the height of this ControlPanel, in pixels
		 *	@param	width	the width of this ControlPanel, in pixels
		 */
		private ControlPanel(int height, int width)
		{
			// ensure buttons are laid out from left to right
			setLayout(new FlowLayout());
			
			// set size of the panel
			setPreferredSize(new Dimension(height, width));
			
			// instantiate button event handler
			buttonHandler = new ButtonHandler();
			
			// initialize internal components
			initComponents();
		}
		
		
		/**
		 *	Initialize and place internal components.
		 */
		private void initComponents()
		{
			// create Load Puzzle button
			btn_loadPuzzle = new JButton("Load Puzzle");
			btn_loadPuzzle.setFont(fnt_buttons);
			btn_loadPuzzle.setName("Load Puzzle");
			btn_loadPuzzle.addActionListener(buttonHandler);
			
			// create radio button for choosing Brute Force variable selector
			rbtn_bruteForceVariableSelector = new JRadioButton("Brute Force");
			rbtn_bruteForceVariableSelector.setFont(fnt_buttons);
			rbtn_bruteForceVariableSelector.setName("BruteForceVariableSelector");
			rbtn_bruteForceVariableSelector.addActionListener(buttonHandler);
			
			// create radio button for choosing variable selector that uses MRV heuristic
			rbtn_MRVVariableSelector = new JRadioButton("MRV Heuristic");
			rbtn_MRVVariableSelector.setFont(fnt_buttons);
			rbtn_MRVVariableSelector.setName("MRV");
			rbtn_MRVVariableSelector.addActionListener(buttonHandler);
			
			// create button group to enforce only one variable selector selection
			bgrp_variableSelector = new ButtonGroup();
			bgrp_variableSelector.add(rbtn_bruteForceVariableSelector);
			bgrp_variableSelector.add(rbtn_MRVVariableSelector);
			
			// create Solve Puzzle button
			btn_solvePuzzle = new JButton("Solve Puzzle");
			btn_solvePuzzle.setFont(fnt_buttons);
			btn_solvePuzzle.setName("Solve Puzzle");
			btn_solvePuzzle.addActionListener(buttonHandler);
			btn_solvePuzzle.setEnabled(false);
			
			// create Reset Puzzle button
			btn_resetPuzzle = new JButton("Reset");
			btn_resetPuzzle.setFont(fnt_buttons);
			btn_resetPuzzle.setName("Reset Puzzle");
			btn_resetPuzzle.addActionListener(buttonHandler);
			btn_resetPuzzle.setEnabled(false);
			
			// create Cancel Search button
			btn_cancelSearch = new JButton("Cancel");
			btn_cancelSearch.setFont(fnt_buttons);
			btn_cancelSearch.setName("Cancel Search");
			btn_cancelSearch.addActionListener(buttonHandler);
			btn_cancelSearch.setEnabled(false);
			
			// add buttons to this ControlPanel
			this.add(btn_loadPuzzle);
			this.add(rbtn_bruteForceVariableSelector);
			this.add(rbtn_MRVVariableSelector);
			this.add(btn_solvePuzzle);
			this.add(btn_cancelSearch);
			this.add(btn_resetPuzzle);
		}
		
		
		/**
		 *	Inner class ButtonHandler.
		 *	<p>
		 *	This class encapsulates the logic for handling the ControlPanel's button<br>
		 *	press events.<br>
		 *	<p>
		 *	Written by:	Roger West
		 *				University of Illinois at Springfield
		 *				Jun 26, 2014
		 */
		class ButtonHandler implements ActionListener
		{
			
			///////////////////////////////////////////////////////////////////////////////////////////////
			//	Methods
			///////////////////////////////////////////////////////////////////////////////////////////////
			
			/**
			 *	Respond to a button press.
			 *
			 *	@param	e	the ActionEvent associated with the button that was pressed
			 */
			public void actionPerformed(ActionEvent e)
			{
				// get the button that was pressed
				AbstractButton b = (AbstractButton)e.getSource();
				
				// fire appropriate event
				if (b.getName().equals("Load Puzzle"))
				{
					// prompt user for puzzle file to load
					JFileChooser jfc = new JFileChooser();
					jfc.setCurrentDirectory(new File(System.getProperty("user.dir")));
					jfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
					int result = jfc.showOpenDialog(getParent());
					
					if (jfc.getSelectedFile() != null)
					{
						// read puzzle data from selected file
						SudokuPuzzleReader spr = new SudokuPuzzleReader();
						
						puzzle = spr.readPuzzleFromFile(jfc.getSelectedFile());
						
						// save initial state to allow puzzle to be reset
						initialState = puzzle.getCopy();
						
						// perform simple check to ensure puzzle is a 9x9 puzzle
						if (puzzle.getWidth() == MainFrame.this.puzzleWidth && puzzle.getHeight() == MainFrame.this.puzzleHeight)
						{
							// clear the puzzle grid
							MainFrame.this.pnl_puzzle.clearPuzzle();
							
							// valid dimension; display initial state of puzzle
							MainFrame.this.setPuzzle(puzzle);
							
							// clear status bar of any previous information
							MainFrame.this.clearStatus();
							
							// enable Solve Puzzle button if a variable selector was chosen prior to loading a puzzle
							if (bgrp_variableSelector.getSelection() != null)
							{
								btn_solvePuzzle.setEnabled(true);
							}
							
							// disable the Reset button
							ControlPanel.this.btn_resetPuzzle.setEnabled(false);
						}
						else
						{
							// inform user puzzle is chosen file is not a 9x9 puzzle
							JOptionPane.showMessageDialog(null,
														  "Cannot load puzzle from " + jfc.getSelectedFile().getAbsolutePath() + " - puzzle dimensions invalid",
														  "Puzzle Error",
														  JOptionPane.ERROR_MESSAGE);
						}
					}
				}
				else if (b.getName().equals("BruteForceVariableSelector"))
				{
					// set to use default variable selector
					
					// if puzzle has been loaded, enable Solve Puzzle button
					if (MainFrame.this.puzzle != null)
					{
						btn_solvePuzzle.setEnabled(true);
					}
				}
				else if (b.getName().equals("MRV"))
				{
					// set to use variable selector using the MRV heuristic
					
					// if puzzle has been loaded, enable Solve Puzzle button
					if (MainFrame.this.puzzle != null)
					{
						btn_solvePuzzle.setEnabled(true);
					}
				}
				else if (b.getName().equals("Solve Puzzle"))
				{
					// potentially buggy to initialize with a null value, but should be okay
					VariableSelector variableSelector = null;
					
					if (rbtn_bruteForceVariableSelector.isSelected())
					{
						// create order in which to select variables for assignment
						LinkedList<SudokuVariable> variableSelectionOrder = new LinkedList<SudokuVariable>();
						
						for (SudokuVariable[] row : puzzle.getAllCells())
						{
							for (SudokuVariable cell : row)
							{
								variableSelectionOrder.add(cell);
							}
						}
						
						// instantiate BruteForceVariableSelector
						variableSelector = new BruteForceVariableSelector(variableSelectionOrder);
					}
					else if (rbtn_MRVVariableSelector.isSelected())
					{
						// instantiate MRVVariableSelector
						variableSelector = new MRVVariableSelector();
					}
					
					// create the BacktrackingSearch object to be used
					BacktrackingSearch search = new BacktrackingSearch(puzzle, variableSelector);
					
					// create the agent that will solve the puzzle
					SudokuCSPAgent agent = new SudokuCSPAgent(puzzle, search);
					
					// run agent on separate SwingWorker thread to ensure synchronous GUI updates
					// as search proceeds
					worker = new SudokuWorker(agent);
					worker.execute();
					
					// enable the Cancel Search button
					ControlPanel.this.btn_cancelSearch.setEnabled(true);
				}
				else if (b.getName().equals("Cancel Search"))
				{
					// stop agent
					worker.cancel(true);
					
					// disable the Cancel Search button
					b.setEnabled(false);
				}
				else if (b.getName().equals("Reset Puzzle"))
				{
					// clear grid
					MainFrame.this.pnl_puzzle.clearPuzzle();
					
					// Reset the working copy of the current puzzle to its initial state
					MainFrame.this.puzzle = MainFrame.this.initialState;
					
					// reassign initial state
					MainFrame.this.initialState = puzzle.getCopy();
					
					// set puzzle
					MainFrame.this.setPuzzle(puzzle);
					
					// clear the status bar
					MainFrame.this.clearStatus();
					
					// disable the Reset button
					b.setEnabled(false);
				}
			}
			
			
			/**
			 *	Inner class SudokuWorker.
			 *	<p>
			 *	This class serves to allow the agent to solve puzzle in the background, but still<br>
			 *	allow GUI to be updated to show the progress of the search.<br>
			 *	<p>
			 *	Written by:	Roger West
			 *				University of Illinois at Springfield
			 *				Jun 26, 2014
			 */
			class SudokuWorker extends SwingWorker<Boolean, Void>
			{
				
				///////////////////////////////////////////////////////////////////////////////////////////////
				//	Attributes
				///////////////////////////////////////////////////////////////////////////////////////////////
				
				/** The agent that will use this thread. */
				private SudokuCSPAgent agent;
				
				
				///////////////////////////////////////////////////////////////////////////////////////////////
				//	Constructors
				///////////////////////////////////////////////////////////////////////////////////////////////
				
				/**
				 *	Create a new SudokuWorker thread.
				 *
				 *	@param	agent	the agent that will run on this thread
				 */
				SudokuWorker(SudokuCSPAgent agent)
				{
					this.agent = agent;
				}

				@Override
				/**
				 *	Solve the puzzle in the background.
				 */
				protected Boolean doInBackground() throws Exception
				{
					try
					{
						// instruct agent to solve puzzle, and update the main application window
						agent.solveProblem(MainFrame.this);
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
					
					// return true to indicate agent is done
					return true;
				}

				@Override
				/**
				 *	Agent is done solving puzzle. Update states of GUI components accordingly.
				 */
				protected void done()
				{
					ControlPanel.this.btn_resetPuzzle.setEnabled(true);
					ControlPanel.this.btn_cancelSearch.setEnabled(false);
				}
			}
		}
	}

	
	/**
	 *	Inner class StatusBar.
	 *	<p>
	 *	This class serves as a text field at the bottom of the main application window where<br>
	 *	information specific to the Sudoku puzzle being solved may be displayed.<br>
	 *	<p>
	 *	Written by:	Roger West
	 *				University of Illinois at Springfield
	 *				Jun 26, 2014
	 */
	class StatusBar extends JPanel
	{
		
		///////////////////////////////////////////////////////////////////////////////////////////////
		//	Attributes
		///////////////////////////////////////////////////////////////////////////////////////////////
		
		/** Text field for displaying information. */
		private JLabel lbl_status;
		
		/** Font for status text */
		private Font fnt_statusText = new Font("Calibri", Font.BOLD, 14);
		
		
		///////////////////////////////////////////////////////////////////////////////////////////////
		//	Constructors
		///////////////////////////////////////////////////////////////////////////////////////////////
		
		/**
		 *	Create a new StatusBar with the specified dimensions, in pixels.
		 *
		 *	@param	preferredSize	the size, in pixels, to build this StatusBar
		 */
		private StatusBar(Dimension preferredSize)
		{
			super();
			
			// set size
			setPreferredSize(preferredSize);
			
			// set border to give look and feel of a typical status bar
			setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
			
			// use standard BorderLayout
			setLayout(new BorderLayout());
			
			// initialize internal components
			initComponents();
		}
		
		
		///////////////////////////////////////////////////////////////////////////////////////////////
		//	Methods
		///////////////////////////////////////////////////////////////////////////////////////////////
		
		/**
		 *	Initialize internal components.
		 */
		private void initComponents()
		{
			// create and place lone text field on left side of status bar
			lbl_status = new JLabel();
			lbl_status.setFont(fnt_statusText);
			
			this.add(lbl_status, BorderLayout.WEST);
		}
		
		
		/**
		 *	Update the text being displayed by this StatusBar.
		 *
		 *	@param	newStatus	the text to display
		 */
		private void updateStatus(String newStatus)
		{
			lbl_status.setText(newStatus);
		}
	}
}