package gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JPanel;

/**
 *	Class PuzzlePanel.
 *	<p>
 *	This class serves as a container panel for the array of Cell panels that comprise the<br>
 *	grid of a Sudoku puzzle.<br>
 *	<p>
 *	Written by:	Roger West
 *				University of Illinois at Springfield
 *				Jun 26, 2014
 */
public class PuzzlePanel extends JPanel
{
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Attributes
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/** Height of this PuzzlePanel, in terms of the number of rows in a Sudoku puzzle. */
	private int height;
	
	/** Width of this PuzzlePanel, in terms of the number of columns in a Sudoku puzzle. */
	private int width;

	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Constructors
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Create a new PuzzlePanel with the specified height and width.
	 *
	 *	@param	height		the height, in terms of the number of rows
	 *	@param	width		the width, in terms of the number of columns
	 */
	public PuzzlePanel(int height, int width)
	{
		// save height & width values
		this.height = height;
		this.width = width;
		
		// use a GridLayout for the individual Cells
		setLayout(new GridLayout(height, width));
		
		// set the size of this PuzzlePanel, in pixels
		setPreferredSize(new Dimension(height * Cell.DEFAULT_HEIGHT, width * Cell.DEFAULT_WIDTH));
	}
	

	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Methods
	///////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 *	Add the specified Cell to the grid of this PuzzlePanel.
	 *
	 *	@param	newCell		the Cell to add
	 */
	public void addCell(Cell newCell)
	{
		this.add(newCell);
	}
	
	
	/**
	 *	Clear the entire puzzle grid. All Cells are set to be unassigned.
	 */
	public void clearPuzzle()
	{
		for (int row = 0; row < height; row++)
		{
			for (int column = 0; column < width; column++)
			{
				getCellByName("" + row + "," + column).resetCell();
			}
		}
	}
	
	
	/**
	 *	Reset the puzzle grid to the puzzle's initial state. All Cells that were assigned
	 *	values by a search will be unassigned.
	 */
	public void resetPuzzle()
	{
		Cell currentCell;
		
		for (int row = 0; row < height; row++)
		{
			for (int column = 0; column < width; column++)
			{
				currentCell = getCellByName("" + row + "," + column); 
				
				if (currentCell.wasSearchAssigned())
				{
					currentCell.setValueAsUnassigned();
				}
			}
		}
	}
	
	
	/**
	 *	Set the value for the Cell at the specified row and column.
	 *
	 *	@param	row		row position for the Cell to set
	 *	@param	column	column position for the Cell to set
	 *	@param	value	the value to set for the Cell
	 */
	public void setCellValue(int row, int column, String value)
	{
		// retrieve Cell by name
		Cell cell = getCellByName("" + row + "," + column);
		
		if (cell != null)
		{
			// cell found; set value
			cell.setValue(value);
		}
		else
		{
			// cell not found; should not happen
			System.out.println("Cell " + row + "," + column + " is NULL");
		}
	}
	
	
	/**
	 *	Return the Cell located at the specified row & column.
	 *
	 *	@param	row		the row position of the Cell to return
	 *	@param	column	the column position of the Cell to return
	 *
	 *	@return	the Cell located at <row,column>
	 */
	public Cell getCellByRowColumn(int row, int column)
	{
		return getCellByName("" + row + "," + column);
	}
	
	
	/**
	 *	Return the Cell with the specified name.
	 *
	 *	@param	name	the name identifying the Cell to return
	 *
	 *	@return	the Cell whose name property equals the specified name
	 */
	public Cell getCellByName(String name)
	{
		// potentially buggy to initialize as null
		Cell c = null;
		
		// must traverse all Components in this PuzzlePanel to find the Cell
		for (Component comp : this.getComponents())
		{
			if (comp instanceof Cell)
			{
				if (((Cell)comp).getName().equals(name))
				{
					return (Cell)comp;
				}
			}
		}
		
		return c;
	}
	
}
