package sudokuCSP;

import gui.MainFrame;

/**
 *	Class BacktrackingSearch.
 *	<p>
 *	This class represents a simple depth-first search for solving a CSP. The<br>
 *	variables of the CSP are assigned one at a time, and the search backtracks<br>
 *	if an inconsistent assignment is reached, or if the current variable has no<br>
 *	legal values left to assign.<br>
 *	<p>
 *	This class is designed to work with the MainFrame class to show the progress of<br>
 *	the search, and to show the number of variable assignments that have been tried.<br>
 *	This coupling between the search algorithm and the GUI is technically a design flaw,<br>
 *	but for this small program it is tolerable.<br>
 *	<p>
 *	Written by:	Roger West<br>
 *				University of Illinois at Springfield<br>
 *				Jun 26, 2014<br>
 *
 */
public class BacktrackingSearch
{
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Attributes
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/** The algorithm for selecting the next variable to assign. */
	private VariableSelector variableSelector;
	
	/** The running total number of variable assignments that have been attempted. This value
	 *	includes assignments that later prove to be incorrect.
	 */
	private long numVariableAssignments;
	

	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Constructors
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Create a new BacktrackingSearch.
	 *
	 *	@param	puzzle				the SudokuPuzzle being solved
	 *	@param	variableSelector	the algorithm for selecting the next variable to assign 
	 */
	public BacktrackingSearch(SudokuPuzzle puzzle, VariableSelector variableSelector)
	{
		this.variableSelector = variableSelector;
	}

	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Methods
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Perform the search, given the specified initial state of the puzzle, and
	 *	the specified reference to the main program window for displaying the progress
	 *	of the search.
	 *
	 *	@param	initialState	the initial puzzle assignment
	 *	@param	mainFrame		reference to the main program window for updates
	 *
	 *	@return		the complete and consistent assignment found by the search;
	 *				null if the search did not find an assignment that was both complete and consistent
	 */
	public SudokuPuzzle search(SudokuPuzzle initialState, MainFrame mainFrame)
	{
		// initially, no variable assignments have been tried
		numVariableAssignments = 0;
		
		// initiate recursive search
		return recursiveAssignment(initialState, mainFrame);
	}

	
	/**
	 *	Recursive helper method that performs the bulk of the work for the search.
	 *
	 *	@param	workingCopy		current assignment of the SudokuPuzzle being solved
	 *	@param	mainFrame		reference to the main program window for updates
	 */
	private SudokuPuzzle recursiveAssignment(SudokuPuzzle workingCopy, MainFrame mainFrame)
	{
		// first check to see if search was cancelled by user
		if (mainFrame.getAgentWorker().isCancelled())
		{
			// search was cancelled; return
			return null;
		}
		
		// check for solution
		if (workingCopy.isComplete())
		{
			// solution found (note that inconsistencies will be filtered out prior to this point)
			return workingCopy;
		}
		
		// select next variable to assign
		SudokuVariable nextVariable = variableSelector.getNextUnassignedVariable(workingCopy);
		
		// increment number of assignments attempted, and update GUI
		numVariableAssignments++;
		mainFrame.updateStatus("# Variable assignments attempted: " + numVariableAssignments);

		// this changes the background color of the current cell so backtracking can be visualized
		mainFrame.setCellAsCurrent(nextVariable.getRow(), nextVariable.getColumn());
		
		// cycle through legal values
		for (String currentValue : workingCopy.getDomain())
		{
			// attempt assignment of current value to current variable
			workingCopy.assignVariable(nextVariable, currentValue);
			
			// update GUI
			mainFrame.setCellValue(nextVariable.getRow(), nextVariable.getColumn(), currentValue);
			
			// check for consistent assignment
			if (workingCopy.isConsistent())
			{	
				// assignment is consistent, so try to proceed
				mainFrame.setCellAsNotCurrent(nextVariable.getRow(), nextVariable.getColumn());
				
				SudokuPuzzle result = recursiveAssignment(workingCopy, mainFrame);
				
				if (result != null)
				{
					// assignment is consistent
					return result;
				}
				
				// assignment not consistent; remove assignment
				workingCopy.unassignVariable(nextVariable);
				
				// update GUI
				mainFrame.setCellAsUnassigned(nextVariable.getRow(), nextVariable.getColumn());
				mainFrame.setCellAsNotCurrent(nextVariable.getRow(), nextVariable.getColumn());
			}
			else
			{
				// assignment not consistent; remove last assignment
				workingCopy.unassignVariable(nextVariable);
				
				// update GUI
				mainFrame.setCellAsUnassigned(nextVariable.getRow(), nextVariable.getColumn());
				mainFrame.setCellAsNotCurrent(nextVariable.getRow(), nextVariable.getColumn());
			}
		}
		
		// failure to find a solution
		return null;
	}
}
