package sudokuCSP;

import java.util.Iterator;
import java.util.List;

/**
 *	Class BruteForceVariableSelector.
 *	<p>
 *	This class represents a brute force method for selecting the next unassigned Variable<br>
 *	for a SudokuPuzzle. The brute force method always begins by systematically starting at the<br>
 *	cell in the upper left corner, and proceeding left to right, top to bottom, to find the next<br>
 *	unasigned cell.<br>
 *	<p>
 *	Written by:	Roger West<br>
 *				University of Illinois at Springfield<br>
 *				Jun 26, 2014<br>
 *
 */
public class BruteForceVariableSelector implements VariableSelector
{
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Attributes
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/** The order in which variables will be selected for assignment. */
	private List<SudokuVariable> assignmentOrder;
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Constructors
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Create a new DefaultVariableSelector with the ordering defined in the
	 *	specified List.
	 *
	 *	@param	order	order in which variables will be tried
	 */
	public BruteForceVariableSelector(List<SudokuVariable> order)
	{
		assignmentOrder = order;
	}
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Methods
	///////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 *	Return the next unassigned Variable for the specified SudokuPuzzle.
	 *
	 *	@param	puzzle		the SudokuPuzzle being solved
	 * 
	 *	@return	the next unassigned Variable, according to the ordering for this
	 *			BruteForceVariableSelector
	 */
	public SudokuVariable getNextUnassignedVariable(SudokuPuzzle puzzle)
	{
		// the next unassigned variable
		SudokuVariable nextVariable;
		
		// search for the first unassigned variable
		for (Iterator<SudokuVariable> itr = assignmentOrder.iterator(); itr.hasNext(); )
		{
			// get current Variable
			nextVariable = itr.next();
			
			if (nextVariable.getValue().equals(SudokuPuzzle.EMPTY_CELL_VALUE))
			{
				// vName is not assigned
				return nextVariable;
			}
		}
			
		// all variables assigned
		return null;
	}

	/**
	 *	Return the order for the variable assignments.
	 *
	 *	@return	the variable assignment order
	 */
	public List<SudokuVariable> getAssignmentOrder()
	{
		return assignmentOrder;
	}

	
	/**
	 *	Set the variable assignment order for this BruteForceVariableSelector.
	 *
	 *	@param	itemOrder		the variable assignment order
	 */
	public void setAssignmentOrder(List<SudokuVariable> assignmentOrder)
	{
		this.assignmentOrder = assignmentOrder;
	}
	
	
	/**
	 *	Return a String representation of this BruteForceVariableSelector.
	 */
	public String toString()
	{
		return assignmentOrder.toString();
	}

}
