package sudokuCSP;

import java.util.HashSet;

/**
 *	Class SudokuMRVVariableSelector.
 *	<p>
 *	This class represents an implementation of the Minimum Remaining Values (MRV)<br>
 *	heuristic for selecting the next variable to assign in a Sudoku puzzle. The<br>
 *	basic premise of the MRV heuristic is that the effective size of the search space<br>
 *	will be greatly reduced if the cell (variable) with the fewest legal values remaining<br>
 *	is assigned next. There are three aspects to this with regard to a Sudoku puzzle:<br>
 *	<p>
 *		1.	the minimum remaining values in the box a cell is in<br>
 *		2.	the minimum remaining values in the column a cell is in<br>
 *		3.	the minimum remaining values in the row a cell is in<br>
 *	<p>
 *	This algorithm computes the minimum remaining values for each unassigned cell by<br>
 *	tallying the number of unique, already assigned values in the row, column, and box of<br>
 *	which each cell resides, and subtracting this value from the size of the domain of<br>
 *	legal values.<br>
 *	<p>
 *	Written by:	Roger West<br>
 *				University of Illinois at Springfield<br>
 *				Jun 26, 2014<br>
 *
 */
public class MRVVariableSelector implements VariableSelector
{

	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Constructors
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Create a new MRVVariableSelector.
	 */
	public MRVVariableSelector()
	{
		
	}
	

	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Methods
	///////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 *	Return the next SudokuVariable to assign for the specified SudokuPuzzle.
	 *	The next SudokuVariable to be assigned will be the SudokuVariable with
	 *	the fewest remaining legal values. In the event more than one variable has
	 *	the same MRV value, the first variable found with that value will be the one
	 *	that is returned.
	 *
	 *	@param	puzzle		the Sudoku puzzle being solved
	 * 
	 *	@return	the next unassigned Variable
	 */
	public SudokuVariable getNextUnassignedVariable(SudokuPuzzle puzzle)
	{
		// potentially buggy to initialize with null value, but returning the null should never happen
		SudokuVariable nextVariable = null;
		
		// the current lowest MRV value
		int lowestMRV = 9;
		
		// calculate MRV values for all variables (cells); keep track of the one with the
		// lowest MRV value
		for (int row = 0; row < puzzle.getAllCells().length; row++)
		{
			for (int column = 0; column < puzzle.getAllCells().length; column++)
			{
				if (puzzle.getAllCells()[row][column].getValue().equals(SudokuPuzzle.EMPTY_CELL_VALUE))
				{
					// calculate MRV value for current cell
					int currentMRV = getRemainingValues(puzzle.getAllCells()[row][column], puzzle);

					if (currentMRV < lowestMRV)
					{
						// new lowest MRV value found; update variable to return
						lowestMRV = currentMRV;
						nextVariable = puzzle.getAllCells()[row][column];
					}
				}
			}
		}
		
		// return next Variable to assign
		return nextVariable;
	}
	
	
	/**
	 *	Calculate the number of remaining values for the specified SudokuVariable,
	 *	and given the specified puzzle state.
	 *
	 *	@param	variable	the SudokuVariable being processed
	 *	@param	puzzle		the SudokuPuzzle being solved
	 *
	 *	@return	an int in the range 0-9, representing the number of domain values
	 *			that could possibly be assigned to the specified SudokuVariable
	 */
	private int getRemainingValues(SudokuVariable variable, SudokuPuzzle puzzle)
	{
		// use a HashSet for O(1) duplicate check
		HashSet<String> assignedValues = new HashSet<String>();
		
		// tally already assigned values for variable's row
		for (SudokuVariable var : puzzle.getRow(variable.getRow()))
		{
			if (!var.getValue().equals(SudokuPuzzle.EMPTY_CELL_VALUE))
			{
				assignedValues.add(var.getValue());
			}
		}
		
		// tally already assigned values for variable's column
		for (SudokuVariable var : puzzle.getColumn(variable.getColumn()))
		{
			if (!var.getValue().equals(SudokuPuzzle.EMPTY_CELL_VALUE))
			{
				assignedValues.add(var.getValue());
			}
		}
		
		// tally already assigned values for variable's box
		for (SudokuVariable var : puzzle.getBox(variable.getRow(), variable.getColumn()))
		{
			if (!var.getValue().equals(SudokuPuzzle.EMPTY_CELL_VALUE))
			{
				assignedValues.add(var.getValue());
			}
		}
		
		// number of remaining values == domain size - # of already assigned values
		return puzzle.getDomain().length - assignedValues.size();
	}

}
