package sudokuCSP;

import java.util.HashSet;

/**
 *	Class SudokuPuzzle.
 *	<p>
 *	This class represents a task environment for a 9x9 Sudoku puzzle, to be solved as<br>
 *	a constraint satisfaction problem (CSP). The environment consists of a square grid<br>
 *	of cells, 9 cells wide and 9 cells high. The grid is subdivided into 3 types of<br>
 *	structures:<br>
 *		1)	rows<br>
 *		2)	columns<br>
 *		3)	boxes, which are contiguous 3x3 subgrids, located as follows:<br>
 *			a)	top left, top center, top right<br>
 *			b)	left center, center, right center<br>
 *			c)	bottom left, bottom center, bottom right<br>
 *	<p>
 *	Each cell may contain one and only one character value. The size of the domain of<br>
 *	legal values is equal to the width of the puzzle, so for a 9x9 puzzle the domain<br>
 *	consists of 9 unique values. These can be any values that can be represented as a<br>
 *	String type, though the digits 1-9 are commonly used.<br>
 *	<p>
 *	The goal of the puzzle is to assign values to the cells such that the following 3<br>
 *	constraints hold:<br>
 *		1) Each row contains a different permutation of the 9 legal values<br>
 *		2) Each column contains a different permutation of the 9 legal values<br>
 *		3) Each box contains a different permutation of the 9 legal values<br>
 *	<p>
 *	Typically some of the cells in the grid are pre-assigned, with the number of pre-<br>
 *	assigned values constraining the number of possible solution. Generally, when at<br>
 *	least 17 cells have been pre-assigned, there will only be one solution.<br>
 *	<p>
 *	For the purposes of solving a SudokuPuzzle as a CSP, each cell of the puzzle can<br>
 *	also be modeled as one of the variables in an assignment.<br>
 *	<p>
 *	Written by:	Roger West<br>
 *				University of Illinois at Springfield<br>
 *				Jun 26, 2014<br>
 *
 */
public class SudokuPuzzle
{
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Attributes
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/** Default value for a cell that is not currently assigned a value */
	public final static String EMPTY_CELL_VALUE = "-";
	
	/** The 9x9 grid of cells */
	private SudokuVariable[][] allCells;
	
	/** The domain of legal values. Values will be tried in the order in which they appear in the array. */
	private String[] domain;
	
	/** The width, in number of cells, of this SudokuPuzzle. */
	private int width;
	
	/** The height, in number of cells, of this SudokuPuzzle. */
	private int height;
	
	/** Set of all rows in this SudokuPuzzle */
	private SudokuVariable[][] allRows;
	
	/** Set of all columns in this SudokuPuzzle */
	private SudokuVariable[][] allColumns;
	
	/** Set of all boxes in this SudokuPuzzle */
	private SudokuVariable[][] allBoxes;
	
	/** Set of constraints governing the solving of this SudokuPuzzle */
	private HashSet<SudokuConstraint> constraints;
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Constructors
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Create a new SudokuPuzzle.
	 *
	 *	@param	domain			the domain of legal values
	 *	@param	initialState	the initially assigned grid
	 */
	public SudokuPuzzle(String[] domain,
						SudokuVariable[][] initialState)
	{
		// set puzzle attributes
		this.width = initialState[0].length;
		this.height = initialState.length;
		this.domain = domain;
		this.allCells = initialState;
		
		// store all rows, all columns, and all boxes as separate sets 
		allRows = findAllRows();
		allColumns = findAllColumns();
		allBoxes = findAllBoxes();
		
		// populate set of constraints
		constraints = new HashSet<SudokuConstraint>();
		constraints.add(new RowConstraint());
		constraints.add(new ColumnConstraint());
		constraints.add(new BoxConstraint());
	}
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	//	Methods
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Return the domain of legal values for this SudokuPuzzle.
	 *
	 *	@return	the set of legal values
	 */
	public String[] getDomain()
	{
		return domain;
	}
	
	
	/**
	 *	Return the width of this SudokuPuzzle.
	 *
	 *	@return	puzzle width
	 */
	public int getWidth()
	{
		return width;
	}
	
	
	/**
	 *	Return the height of this SudokuPuzzle.
	 *
	 *	@return	puzzle height
	 */
	public int getHeight()
	{
		return height;
	}
	
	
	/**
	 *	Return the set of all cells of this SudokuPuzzle.
	 *
	 *	@return	a 2D array of SudokuVariables
	 */
	public SudokuVariable[][] getAllCells()
	{
		return allCells;
	}
	
	
	/**
	 *	Return the set of all rows of this SudokuPuzzle.
	 *
	 *	@return	an array in which each element is 1 row of SudokuVariables
	 */
	public SudokuVariable[][] getAllRows()
	{
		return allRows;
	}
	
	
	/**
	 *	Return the set of all columns of this SudokuPuzzle.
	 *
	 *	@return	an array in which each element is 1 column of SudokuVariables
	 */
	public SudokuVariable[][] getAllColumns()
	{
		return allColumns;
	}
	
	
	/**
	 *	Return the set of all boxes of this SudokuPuzzle.
	 *
	 *	@return	an array in which each element is 1 box of SudokuVariables
	 */
	public SudokuVariable[][] getAllBoxes()
	{
		return allBoxes;
	}
	
	
	/**
	 *	Return the row at the specified index.
	 *
	 *	@param	rowNum	an int in the range 0-8, where 0 is the top row and 8 is the bottom row
	 *
	 *	@return	the row of SudokuVariables at the specified position
	 */
	public SudokuVariable[] getRow(int rowNum)
	{
		return allRows[rowNum];
	}
	
	
	/**
	 *	Return the column at the specified index.
	 *
	 *	@param	colNum	an int in the range 0-8, where 0 is the left column and 8 is the right column
	 *
	 *	@return	the column of SudokuVariables at the specified position
	 */
	public SudokuVariable[] getColumn(int colNum)
	{
		return allColumns[colNum];
	}
	
	
	/**
	 *	Return the box containing the cell at the specified row and column.
	 *
	 *	@param	row		an int in the range 0-8, where 0 is the top row and 8 is the bottom row
	 *	@param	column	an int in the range 0-8, where 0 is the left column and 8 is the right column
	 *
	 *	@return	the box of SudokuVariables that contains the cell at <row,column>
	 */
	public SudokuVariable[] getBox(int row, int column)
	{
		if (row <= 2 && column <= 2)		// top left box
		{
			return allBoxes[0];
		}
		else if (row <= 5 && column <= 2)	// top center box
		{
			return allBoxes[1];
		}
		else if (row <= 8 && column <= 2)	// top right box
		{
			return allBoxes[2];
		}
		else if (row <= 2 && column <= 5)	// left center box
		{
			return allBoxes[3];
		}
		else if (row <= 5 && column <= 5)	// center box
		{
			return allBoxes[4];
		}
		else if (row <= 8 && column <= 5)	// right center box
		{
			return allBoxes[5];
		}
		else if (row <= 2 && column <= 8)	// bottom left box
		{
			return allBoxes[6];
		}
		else if (row <= 5 && column <= 8)	// bottom center box
		{
			return allBoxes[7];
		}
		else if (row <= 8 && column <= 8)	// bottom right box
		{
			return allBoxes[8];
		}
		
		// should never reach here
		return null;
	}
	
	
	/**
	 *	Return a copy of this SudokuPuzzle, in its current assignment state. If this
	 *	copy is generated prior to assigning values to any unassigned cells, a copy
	 *	of the puzzle's initial state will be returned.
	 *
	 *	@return	a SudokuPuzzle containing the current cell assignments
	 */
	public SudokuPuzzle getCopy()
	{
		SudokuVariable[][] copy = new SudokuVariable[allCells.length][allCells[0].length];
		
		for (int row = 0; row < allCells.length; row++)
		{
			for (int col = 0; col < allCells[0].length; col++)
			{
				copy[row][col] = new SudokuVariable(row, col, allCells[row][col].getValue());
			}
		}
		
		return new SudokuPuzzle(this.domain, copy);
	}
	

	/**
	 *	Return a 2D array in which each row of the array represents a row in this SudokuPuzzle.
	 *	The purpose of this is to facilitate checking the row constraint.
	 *
	 *	@return	all rows of cells
	 */
	public SudokuVariable[][] findAllRows()
	{
		SudokuVariable[][] allRows = new SudokuVariable[domain.length][domain.length];
		
		for (int row = 0; row < domain.length; row++)
		{
			for (int col = 0; col < domain.length; col++)
			{
				allRows[row][col] = allCells[row][col];
			}
		}
		
		return allRows;
	}
	
	
	/**
	 *	Return a 2D array in which each row of the array represents a column in this SudokuPuzzle.
	 *	The purpose of this is to facilitate checking the column constraint.
	 *
	 *	@return	all columns of cells
	 */
	public SudokuVariable[][] findAllColumns()
	{
		SudokuVariable[][] allColumns = new SudokuVariable[domain.length][domain.length];
		
		for (int row = 0; row < domain.length; row++)
		{
			for (int col = 0; col < domain.length; col++)
			{
				allColumns[row][col] = allCells[col][row];
			}
		}
		
		return allColumns;
	}
	
	
	/**
	 *	Return a 2D array in which each row of the array represents a box in this SudokuPuzzle.
	 *	The purpose of this is to facilitate checking the box constraint.
	 *
	 *	@return	all boxes of cells
	 */
	public SudokuVariable[][] findAllBoxes()
	{
		SudokuVariable[][] allBoxes = {{allCells[0][0], allCells[1][0], allCells[2][0],		// top left box
										allCells[0][1], allCells[1][1], allCells[2][1],
										allCells[0][2], allCells[1][2], allCells[2][2]},
									   {allCells[3][0], allCells[4][0], allCells[5][0],		// top center box
										allCells[3][1], allCells[4][1], allCells[5][1],
										allCells[3][2], allCells[4][2], allCells[5][2]},
									   {allCells[6][0], allCells[7][0], allCells[8][0],		// top right box
										allCells[6][1], allCells[7][1], allCells[8][1],
										allCells[6][2], allCells[7][2], allCells[8][2]},
									   {allCells[0][3], allCells[1][3], allCells[2][3],		// center left box
										allCells[0][4], allCells[1][4], allCells[2][4],
										allCells[0][5], allCells[1][5], allCells[2][5]},
									   {allCells[3][3], allCells[4][3], allCells[5][3],		// center box
										allCells[3][4], allCells[4][4], allCells[5][4],
										allCells[3][5], allCells[4][5], allCells[5][5]},
									   {allCells[6][3], allCells[7][3], allCells[8][3],		// center right box
										allCells[6][4], allCells[7][4], allCells[8][4],
										allCells[6][5], allCells[7][5], allCells[8][5]},
									   {allCells[0][6], allCells[1][6], allCells[2][6],		// bottom left box
										allCells[0][7], allCells[1][7], allCells[2][7],
										allCells[0][8], allCells[1][8], allCells[2][8]},
									   {allCells[3][6], allCells[4][6], allCells[5][6],		// bottom center box
										allCells[3][7], allCells[4][7], allCells[5][7],
										allCells[3][8], allCells[4][8], allCells[5][8]},
									   {allCells[6][6], allCells[7][6], allCells[8][6],		// bottom right box
										allCells[6][7], allCells[7][7], allCells[8][7],
										allCells[6][8], allCells[7][8], allCells[8][8]}
									  };
		
		return allBoxes;
	}
	
	
	/**
	 *	Return the size of this puzzle, in terms of total number of cells.
	 *
	 *	@return	an int equal to the total number of cells in this SudokuPuzzle
	 */
	public int getPuzzleSize()
	{
		return allCells.length * allCells[0].length;
	}
	
	
	/**
	 *	Assign the specified value to the specified SudokuVariable (i.e., cell) of this SudokuPuzzle.
	 *
	 *	@param	variable	the SudokuVariable to assign
	 *	@param	value		the value to assign
	 */
	public void assignVariable(SudokuVariable variable, String value)
	{
		allCells[variable.getRow()][variable.getColumn()].setValue(value);
	}
	
	
	/**
	 *	Remove the assignment of the specified SudokuVariable (i.e., cell) of this SudokuPuzzle.
	 *	Removing an assignment results in setting the value of the SudokuVariable to an empty value.
	 *
	 *	@param	variable	the SudokuVariable to assign
	 */
	public void unassignVariable(SudokuVariable variable)
	{
		allCells[variable.getRow()][variable.getColumn()].setValue(EMPTY_CELL_VALUE);
	}
	
	
	/**
	 *	Return whether or not the current assignment of this SudokuPuzzle is consistent. The assignment
	 *	is consistent if none of the puzzle's constraints are violated.
	 *
	 *	@return	true, if none of the constraints (row, column, or box) are violated
	 *			false, if any of the constraints is violated
	 */
	public boolean isConsistent()
	{
		for (SudokuConstraint constraint : constraints)
		{
			if (!constraint.isSatisfied(this))
			{
				// constraint violated; assignment not consistent
				return false;
			}
		}
		
		// no constraints violated; assignment is consistent
		return true;
	}
	
	
	/**
	 *	Return the number of cells that have been assigned non-empty values.
	 *
	 *	@return	an int in the range 0 to getPuzzleSize(), representing the number
	 *			of cells containing non-empty values
	 */
	public int getNumberOfAssignedCells()
	{
		int numAssignedCells = 0;
		
		for (SudokuVariable[] row : allCells)
		{
			for (SudokuVariable variable : row)
			{
				if (!variable.getValue().equals(EMPTY_CELL_VALUE))
				{
					numAssignedCells++;
				}
			}
		}
		
		return numAssignedCells;
	}
	
	
	/**
	 *	Return whether the current assignment of this SudokuPuzzle is complete. A complete
	 *	assignment is an assignment where all cells in the puzzle have been assigned non-empty
	 *	values.
	 *
	 *	@return	true, if all cells of this SudokuPuzzle are assigned non-empty values
	 *			false, if 1 or more cells contain empty values; i.e., are not assigned
	 */
	public boolean isComplete()
	{
		return getNumberOfAssignedCells() == getPuzzleSize();
	}

	
	/**
	 *	Return a String representation of this SudokuPuzzle.
	 */
	public String toString()
	{
		String s = "";
		
		for (SudokuVariable[] row : allCells)
		{
			for (SudokuVariable variable : row)
			{
				s += variable.getValue().toString();
			}
			
			s += "\n";
		}
		
		return	s;
	}

}
