package sudokuCSP;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import javax.swing.JOptionPane;

/**
 *	Class SudokuPuzzleReader.
 *	<p>
 *	This class facilitates the reading of Sudoku puzzles from data files.<br>
 *	<p>
 *	Written by:	Roger West<br>
 *				University of Illinois at Springfield<br>
 *				Jun 26, 2014<br>
 */
public class SudokuPuzzleReader
{
	
	/**
	 * Create and return a SudokuPuzzle based on the data read from the specified file.
	 * 
	 * Sample file format (comments prefaced by "//" should not appear in data file):
	 * 
	 * 9								// puzzle width; i.e., 9 cells wide
	 * 9								// puzzle height; i.e., 9 cells high
	 * 1,2,3,4,5,6,7,8,9				// domain of legal values, in the order in which they should be tried for assignments
	 * ---8-1---						// remaining lines are puzzle data
	 * -------43
	 * 5--------
	 * ----7-8--
	 * 4-----1--
	 * -2--3----
	 * 6------75
	 * --34-----
	 * ---2--6--
	 * <blank line to indicate EOF>		// blank line to indicate EOF
	 * 
	 * 
	 *	@param	puzzleFile		data file containing Sudoku puzzle information
	 *
	 *	@return	a valid SudokuPuzzle, if puzzleFile is a valid data file
	 *			an empty SudokuPuzzle (empty domain, empty puzzle grid) if puzzleFile contains invalid data 
	 */
	public SudokuPuzzle readPuzzleFromFile(File puzzleFile)
	{
		// puzzle width & height
		int width = 0;
		int height = 0;
		
		// the domain of legal values
		String[] domain = {};
		
		// the initial assignment of the grid
		SudokuVariable[][] initialState = {{}};
		
		// row & column indexes for 2D array of initialState
		int row = 0;
		int column = 0;
		
		// current line read from data file
		String lineRead = "";
		
		try
		{
			// create buffered file reader
			BufferedReader br = new BufferedReader(new FileReader(puzzleFile));
			
			// read width & height values
			width = Integer.parseInt(br.readLine());
			height = Integer.parseInt(br.readLine());
			
			// read the domain of legal values
			domain = readDomain(br.readLine());
			
			// create empty grid for puzzle
			initialState = new SudokuVariable[width][height];
			
			// populate puzzle grid
			while ((lineRead = br.readLine()) != null)
			{
				for (int i = 0; i < lineRead.length(); i++)
				{
					initialState[row][column] = new SudokuVariable(row, column, lineRead.substring(i, i + 1));
					
					column++;
					
					if (column == domain.length)
					{
						row++;
						column = 0;
					}
				}
			}
			
			// done; close buffered reader
			br.close();
		}
		catch (IOException ioe)
		{
			// I/O exception occurred; inform user
			JOptionPane.showMessageDialog(null,
										  "Error occurred reading file " + puzzleFile.getAbsolutePath(),
										  "File I/O Error",
										  JOptionPane.ERROR_MESSAGE);
		}
		
		// return populated SudokuPuzzle
		return new SudokuPuzzle(domain, initialState);
	}
	
	
	/**
	 *	Read the domain of legal values from the comma-delimited String.
	 *
	 *	@param	s	comma-delimited String containing legal domain values
	 *
	 *	@return	a String[] containing the legal domain values
	 */
	private String[] readDomain(String s)
	{
		String[] domain = s.split(",");
		
		return domain;
	}
	
}
